@@ -147,7 +147,11 @@ class BuildSystem(abc.ABC):
         Build systems may use this information to do some post processing and
         provide additional, build system-specific, functionality to the users.
 
+        This function will always be executed from the test's stage directory.
+
         .. versionadded:: 3.5.0
+        .. versionchanged:: 3.5.2
+           The function is executed from the stage directory.
 
         :meta private:
 
@@ -741,7 +745,6 @@ class EasyBuild(BuildSystem):
         self.package_opts = {}
         self.prefix = 'easybuild'
         self._eb_modules = []
-        self._prefix_save = None
 
     def emit_build_commands(self, environ):
         if not self.easyconfigs:
@@ -755,7 +758,6 @@ class EasyBuild(BuildSystem):
 
         prefix = os.path.join(os.getcwd(), self.prefix)
         options = ' '.join(self.options)
-        self._prefix_save = prefix
         return [f'export EASYBUILD_BUILDPATH={prefix}/build',
                 f'export EASYBUILD_INSTALLPATH={prefix}',
                 f'export EASYBUILD_PREFIX={prefix}',
@@ -765,7 +767,8 @@ class EasyBuild(BuildSystem):
     def post_build(self, buildjob):
         # Store the modules generated by EasyBuild
 
-        modulesdir = os.path.join(self._prefix_save, 'modules', 'all')
+        modulesdir = os.path.join(os.getcwd(), self.prefix,
+                                  'modules', 'all')
         with open(buildjob.stdout) as fp:
             out = fp.read()
 
