@@ -13,7 +13,6 @@ __all__ = [
 ]
 
 
-import contextlib
 import functools
 import glob
 import inspect
@@ -24,6 +23,7 @@ import shutil
 
 import reframe.core.environments as env
 import reframe.core.fields as fields
+import reframe.core.hooks as hooks
 import reframe.core.logging as logging
 import reframe.core.runtime as rt
 import reframe.utility as util
@@ -78,52 +78,15 @@ DEPEND_BY_ENV = 2
 DEPEND_FULLY = 3
 
 
-def _run_hooks(name=None):
-    def _deco(func):
-        def hooks(obj, kind):
-            if name is None:
-                hook_name = kind + func.__name__
-            elif name is not None and name.startswith(kind):
-                hook_name = name
-            else:
-                # Just any name that does not exist
-                hook_name = 'xxx'
-
-            func_names = set()
-            disabled_hooks = set()
-            func_list = []
-            for cls in type(obj).mro():
-                if hasattr(cls, '_rfm_disabled_hooks'):
-                    disabled_hooks |= cls._rfm_disabled_hooks
-
-                try:
-                    funcs = cls._rfm_pipeline_hooks.get(hook_name, [])
-                    if any(fn.__name__ in func_names for fn in funcs):
-                        # hook has been overriden
-                        continue
-
-                    func_names |= {fn.__name__ for fn in funcs}
-                    func_list += funcs
-                except AttributeError:
-                    pass
-
-            # Remove the disabled hooks before returning
-            return [fn for fn in func_list
-                    if fn.__name__ not in disabled_hooks]
-
-        '''Run the hooks before and after func.'''
-        @functools.wraps(func)
-        def _fn(obj, *args, **kwargs):
-            for h in hooks(obj, 'pre_'):
-                h(obj)
-
-            func(obj, *args, **kwargs)
-            for h in hooks(obj, 'post_'):
-                h(obj)
-
-        return _fn
-
-    return _deco
+_PIPELINE_STAGES = (
+    '__init__',
+    'setup',
+    'compile', 'compile_wait',
+    'run', 'run_wait',
+    'sanity',
+    'performance',
+    'cleanup'
+)
 
 
 def final(fn):
@@ -179,24 +142,22 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
     '''
 
-    @classmethod
-    def disable_hook(cls, hook_name):
+    def disable_hook(self, hook_name):
         '''Disable pipeline hook by name.
 
         :arg hook_name: The function name of the hook to be disabled.
 
         :meta private:
         '''
-        cls._rfm_disabled_hooks.add(hook_name)
+        self._disabled_hooks.add(hook_name)
 
     @classmethod
     def pipeline_hooks(cls):
         ret = {}
-        for c in cls.mro():
-            if hasattr(c, '_rfm_pipeline_hooks'):
-                for kind, hook in c._rfm_pipeline_hooks.items():
-                    ret.setdefault(kind, [])
-                    ret[kind] += hook
+        for phase, hooks in cls._rfm_pipeline_hooks.items():
+            ret[phase] = []
+            for h in hooks:
+                ret[phase].append(h.fn)
 
         return ret
 
@@ -211,7 +172,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: by this test.
     #:
     #: :type: :class:`List[str]`
-    #: :default: ``None``
+    #: :default: ``required``
     #:
     #: .. note::
     #:     .. versionchanged:: 2.12
@@ -223,7 +184,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:     .. versionchanged:: 3.3
     #:        Default value changed from ``[]`` to ``None``.
     #:
-    valid_prog_environs = variable(typ.List[str], type(None), value=None)
+    #:     .. versionchanged:: 3.6
+    #:        Default value changed from ``None`` to ``required``.
+    valid_prog_environs = variable(typ.List[str])
 
     #: List of systems supported by this test.
     #: The general syntax for systems is ``<sysname>[:<partname>]``.
@@ -236,13 +199,15 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:     .. versionchanged:: 3.3
     #:        Default value changed from ``[]`` to ``None``.
     #:
-    valid_systems = variable(typ.List[str], type(None), value=None)
+    #:     .. versionchanged:: 3.6
+    #:        Default value changed from ``None`` to ``required``.
+    valid_systems = variable(typ.List[str])
 
     #: A detailed description of the test.
     #:
     #: :type: :class:`str`
     #: :default: ``self.name``
-    descr = variable(str, type(None), value=None)
+    descr = variable(str)
 
     #: The path to the source file or source directory of the test.
     #:
@@ -337,7 +302,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:
     #: :type: :class:`str`
     #: :default: ``os.path.join('.', self.name)``
-    executable = variable(str, type(None), value=None)
+    executable = variable(str)
 
     #: List of options to be passed to the :attr:`executable`.
     #:
@@ -586,17 +551,16 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: Refer to the :doc:`ReFrame Tutorials </tutorials>` for concrete usage
     #: examples.
     #:
-    #: If set to :class:`None`, a sanity error will be raised during sanity
-    #: checking.
+    #: If not set a sanity error will be raised during sanity checking.
     #:
     #: :type: A deferrable expression (i.e., the result of a :doc:`sanity
-    #:     function </sanity_functions_reference>`) or :class:`None`
-    #: :default: :class:`None`
+    #:     function </sanity_functions_reference>`)
+    #: :default: :class:`required`
     #:
     #: .. note::
     #:    .. versionchanged:: 2.9
     #:       The default behaviour has changed and it is now considered a
-    #:       sanity failure if this attribute is set to :class:`None`.
+    #:       sanity failure if this attribute is set to :class:`required`.
     #:
     #:       If a test doesn't care about its output, this must be stated
     #:       explicitly as follows:
@@ -604,7 +568,10 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:       ::
     #:
     #:           self.sanity_patterns = sn.assert_true(1)
-    sanity_patterns = variable(_DeferredExpression, type(None), value=None)
+    #:
+    #:    .. versionchanged:: 3.6
+    #:       The default value has changed from ``None`` to ``required``.
+    sanity_patterns = variable(_DeferredExpression)
 
     #: Patterns for verifying the performance of this test.
     #:
@@ -799,7 +766,12 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                         os.path.dirname(inspect.getfile(cls))
                     )
 
-        obj._rfm_init(name, prefix)
+        # Attach the hooks to the pipeline stages
+        for stage in _PIPELINE_STAGES:
+            cls._add_hooks(stage)
+
+        # Initialize the test
+        obj.__rfm_init__(name, prefix)
         return obj
 
     def __init__(self):
@@ -812,6 +784,19 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         else:
             return ''
 
+    @classmethod
+    def _add_hooks(cls, stage):
+        pipeline_hooks = cls._rfm_pipeline_hooks
+        fn = getattr(cls, stage)
+        new_fn = hooks.attach_hooks(pipeline_hooks)(fn)
+        setattr(cls, '_rfm_pipeline_fn_' + stage, new_fn)
+
+    def __getattribute__(self, name):
+        if name in _PIPELINE_STAGES:
+            name = f'_rfm_pipeline_fn_{name}'
+
+        return super().__getattribute__(name)
+
     @classmethod
     def __init_subclass__(cls, *, special=False, pin_prefix=False, **kwargs):
         super().__init_subclass__(**kwargs)
@@ -824,26 +809,24 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 os.path.dirname(inspect.getfile(cls))
             )
 
-    def _rfm_init(self, name=None, prefix=None):
+    def __rfm_init__(self, name=None, prefix=None):
         if name is not None:
             self.name = name
 
         # Pass if descr is a required variable.
-        with contextlib.suppress(AttributeError):
-            if self.descr is None:
-                self.descr = self.name
+        if not hasattr(self, 'descr'):
+            self.descr = self.name
 
         # Pass if the executable is a required variable.
-        with contextlib.suppress(AttributeError):
-            if self.executable is None:
-                self.executable = os.path.join('.', self.name)
+        if not hasattr(self, 'executable'):
+            self.executable = os.path.join('.', self.name)
 
         self._perfvalues = {}
 
         # Static directories of the regression check
         self._prefix = os.path.abspath(prefix)
-        if (not os.path.isdir(os.path.join(self._prefix, self.sourcesdir)) and
-            not osext.is_url(self.sourcesdir)):
+        if (self.sourcesdir == 'src' and
+            not os.path.isdir(os.path.join(self._prefix, self.sourcesdir))):
             self.sourcesdir = None
 
         # Runtime information of the test
@@ -883,7 +866,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             # Just an empty environment
             self._cdt_environ = env.Environment('__rfm_cdt_environ')
 
+        # Disabled hooks
+        self._disabled_hooks = set()
+
     # Export read-only views to interesting fields
+
     @property
     def current_environ(self):
         '''The programming environment that the regression test is currently
@@ -1112,7 +1099,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     def _setup_perf_logging(self):
         self._perf_logger = logging.getperflogger(self)
 
-    @_run_hooks()
     @final
     def setup(self, partition, environ, **job_opts):
         '''The setup phase of the regression test pipeline.
@@ -1162,7 +1148,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         self.logger.debug(f'Cloning URL {url} into stage directory')
         osext.git_clone(self.sourcesdir, self._stagedir)
 
-    @_run_hooks('pre_compile')
     @final
     def compile(self):
         '''The compilation phase of the regression test pipeline.
@@ -1269,7 +1254,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
             self._build_job.submit()
 
-    @_run_hooks('post_compile')
     @final
     def compile_wait(self):
         '''Wait for compilation phase to finish.
@@ -1297,9 +1281,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             raise BuildError(self._build_job.stdout,
                              self._build_job.stderr, self._stagedir)
 
-        self.build_system.post_build(self._build_job)
+        with osext.change_dir(self._stagedir):
+            self.build_system.post_build(self._build_job)
 
-    @_run_hooks('pre_run')
     @final
     def run(self):
         '''The run phase of the regression test pipeline.
@@ -1450,7 +1434,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                                  'please use run_complete() instead')
         return self.run_complete()
 
-    @_run_hooks('post_run')
     @final
     def run_wait(self):
         '''Wait for the run phase of this test to finish.
@@ -1481,12 +1464,10 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                                  'please use run_wait() instead')
         self.run_wait()
 
-    @_run_hooks()
     @final
     def sanity(self):
         self.check_sanity()
 
-    @_run_hooks()
     @final
     def performance(self):
         try:
@@ -1520,11 +1501,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 sn.assert_eq(self.job.exitcode, 0,
                              msg='job exited with exit code {0}')
             ]
-            if self.sanity_patterns is not None:
+            if hasattr(self, 'sanity_patterns'):
                 sanity_patterns.append(self.sanity_patterns)
 
             self.sanity_patterns = sn.all(sanity_patterns)
-        elif self.sanity_patterns is None:
+        elif not hasattr(self, 'sanity_patterns'):
             raise SanityError('sanity_patterns not set')
 
         with osext.change_dir(self._stagedir):
@@ -1654,7 +1635,6 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 else:
                     shutil.copy2(f, self.outputdir)
 
-    @_run_hooks()
     @final
     def cleanup(self, remove_files=False):
         '''The cleanup phase of the regression test pipeline.
@@ -1888,7 +1868,6 @@ class RunOnlyRegressionTest(RegressionTest, special=True):
     module.
     '''
 
-    @_run_hooks()
     def setup(self, partition, environ, **job_opts):
         '''The setup stage of the regression test pipeline.
 
@@ -1914,7 +1893,6 @@ class RunOnlyRegressionTest(RegressionTest, special=True):
         This is a no-op for this type of test.
         '''
 
-    @_run_hooks('pre_run')
     def run(self):
         '''The run phase of the regression test pipeline.
 
@@ -1944,7 +1922,6 @@ class CompileOnlyRegressionTest(RegressionTest, special=True):
     module.
     '''
 
-    @_run_hooks()
     def setup(self, partition, environ, **job_opts):
         '''The setup stage of the regression test pipeline.
 
